# FIXSKY1 - Correlate section of object spectrum against sky spectrum
#          using XCSAO and resubtract sky from object+sky
#          reidentify
# November 28, 2005
# By Doug Mink, SAO Telescope Data Center

procedure fixsky1 (object)

file	object=""	{prompt="Object multispec file"}

bool	intxcor=no	{prompt="Run xcsao interactively (y or n)"}
real	ndev=0		{prompt="Reject past this number of standard deviations"}
real	st_lambda=7000.	{prompt="Starting wavelength in Angstroms or pixels to correlate:}
real	end_lambda=8000. {prompt="Ending wavelength in Angstroms or pixels to correlate"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	char arg
	string templist	# File to which to write list of segments
	string xcout	# File to which to write xcsao output
	file objfile, tempfile, ctempfile, ttempfile, ttempfile1
	string dbfile	# id file created by reref
	string dbfile1	# reref output id file renamed to match correlated file
	int band, ap, objband, skyband, rawband
	char	xcmode	# Correlate in pixel, wavelength, or velocity

	real constant
	real nd
	int im
	bool intact
	char xmode

	objband = 1
	skyband = 3
	rawband = 2
	xcmode="wavelength"

	if (ndev < 1) {
	    nd = 10
	    }
	else {
	    nd = ndev
	    }
	intact = intdisp
	objfile = object
	ap = objap
	band = objband
	templist = tempseg
	ctempfile = caltemp
	ttempfile = calshift
	xmode = xcmode

# Cross-correlate spectrum against template, producing output in xcout
	rvsao (motd=no)
	xcsao.spectra = object
	xcsao.specnum = ""
	xcsao.specband = 1
	xcsao.specdir = ""
	xcsao.correlate = "wavelength"
	xcsao.templates = object
	xcsao.tempnum = ""
	xcsao.tempband = 3
	xcsao.tempdir = ""
	xcsao.echelle = no
	xcsao.st_lambda = st_lambda
	xcsao.end_lambda = end_lambda
	xcsao.limfile = ""
	xcsao.obj_plot = no
	xcsao.xcor_plot = no
	xcsao.xcor_file = no
	xcsao.fixbad = yes
	xcsao.badlines = "badlines.dat"
	xcsao.s_emchop = "no"
	xcsao.t_emchop = "no"
	xcsao.s_abs_reject = 0.
	xcsao.s_em_reject = 0.
	xcsao.t_abs_reject = 0.
	xcsao.t_em_reject = 0.
	xcsao.bell_window = 0.05
	xcsao.renormalize = no
	if (end_lambda - st_lambda < 2048.0) {
	    xcsao.ncols = 2048
	    xcsao.top_nrun = 2048
	    xcsao.nrun = 2048
	    }
	else if (end_lambda - st_lambda < 4096.0) {
	    xcsao.ncols = 4096
	    xcsao.top_nrun = 4096
	    xcsao.nrun = 4096
	    }
	else {
	    xcsao.ncols = 8192
	    xcsao.top_nrun = 8192
	    xcsao.nrun = 8192
	    }
	xcsao.interp_mode = "spline3"
	xcsao.zeropad = "yes"
	xcsao.low_bin = 5
	xcsao.top_low = 20
	xcsao.vel_init = "zero"
	xcsao.czguess = 0.
	xcsao.nzpass = 0
	xcsao.tshift = 0.
	xcsao.svel_corr = "none"
	xcsao.tvel_corr = "none"
	xcsao.pkmode = 1
	xcsao.pkfrac = 0.5
	xcsao.pksrch = 25
	xcsao.minvel = -500.
	xcsao.maxvel = 500.
	xcsao.report_mode = 1
	xcsao.logfiles = "xcsao.log,STDOUT"
	xcsao.save_vel = no
	xcsao.rvcheck = no
	xcsao.archive = no
	xcsao.nsmooth = 5
	xcsao.cvel = 0.
	xcsao.dvel = INDEF
	xcsao.ablines = "ablines.dat"
	xcsao.emlines = "emlines.dat"
	xcsao.linedir = "rvsao$lib/"
	xcsao.dispmode = 1
	xcsao.displot = yes
	xcsao.device = "stdgraph"
	xcsao.vel_plot = "correlation"
	xcsao.curmode = intxcor
	xcsao.hardcopy = no
	xcsao.plotter = "stdplot"
	xcsao.temp_plot = no
	xcsao.contsub_plot = no
		xcsao.apodize_plot = no
	xcsao.fft_plot = no
	xcsao.tfft_plot = no
	xcsao.uxcor_plot = no

	xcout = mktemp ("xcsao")
	if (debug) {
	    print ("xcsao(",objfile,",specnum=",ap,",templates=",arg,",tempband=",band,",correlate=",xmode,",ncols=",nbin)
	    }
	xcsao (objfile,specnum=ap,templates=arg,tempband=band,correlate=xmode,ncols=nbin,report_mode=16,logfile=xcout,curmode=intxcor,debug=debug)
	if (debug) {
	    print ("xcsao output written to ",xcout)
	    }

# If spectrum file does not exist for shifted template, create it
	if (!access (ttempfile)) {
	    ttempfile1 = ttempfile // ".fits"
	    if (!access (ttempfile1)) {
		printf ("Fake template file %s does not exist\n", ttempfile)
#		imcopy (ctempfile, ttempfile)
		}
	    }
	else if (debug) {
	    printf ("Fake template file is %s\n",ttempfile)
	    }

# Read pixels and shifts from xcout and fit a curve to them, then make
# a new database/id file with modified pixel positions for lines
	
	reref (objfile, ctempfile, ttempfile, rmin=rmin, smin=nd, shiftfile=xcout, verbose=verbose, debug=debug)

# Rename id file from cross-correlation
	dbfile = "database/id" // ttempfile
	dbfile1 = "database/id" // objfile
	if (debug) {
	    printf ("%s -> %s\n", dbfile, dbfile1)
	    }
	rename (dbfile, dbfile1)

	delete (xcout)
end

# Jul 21 2003	New task
